<!DOCTYPE html>
<html lang="en">
<head>
    <title>Hello World</title>
</head>
<body>
<h1>Hello {{ view.name }}!</h1>
<!-- Start Form 1 -->
<form method="POST"
      action="{{ request.current_route_url() }}">
    <input name="new_name"/>
    <input type="submit" name="form.edit" value="Save"/>
    <input type="submit" name="form.delete" value="Delete"/>
</form>
<!-- End Form 1 -->
</body>
</html>